/*
delhsh.c Version 0.2.0  Hash build test database
Copyright (C) 2003-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Usage: delhsh filename */

#include <bthash.h>
#include <math.h>

#define MAXLMT 999999

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage:   %s filename "
      "#records\n", pgm);
   fprintf(stderr,"Example: %s test.hsh 100\n", pgm);
   fprintf(stderr,"#records is 1 - %d\n",
      MAXLMT);
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int opnerr;
   int i;
   int rslt;
   int lmt;
   int rcdtot;
   int ovfltot;
   int emptytot;
   int freeblk;
   int count;
   int smlen;
   int smlensq;
   int freelen;
   double avg;
   double variance;
   double stdev;
   int delerr;
   int totdel;
   int maxdel;
   int position;
   hashfmt *hshhdr;
   unsigned char *rndSeed;
   unsigned char *dellst;
   unsigned char *lstptr;
   unsigned char *p,*q,*r;
   char dbname[256];
   if (argc != 3) putstx(*argv);
   strcpy(dbname,*(argv+1));
   lmt = (int) atoi(*(argv+2));
   if (lmt < 1 || lmt > MAXLMT) putstx(*argv);

   rndSeed = (unsigned char *) rndinit();
   if (rndSeed == NULL)
      {
      fprintf(stderr,"delhsh: rndinit error\n");
      exit(1);
      } /* rndinit err */

   hshhdr = (hashfmt *) hopen(dbname,UPDT,LOCK,&opnerr);
   if (hshhdr == NULL)
      {
      fprintf(stderr,"delhsh: open "
	 "error %d\n", opnerr);
      free(rndSeed);
      return(0);
      } /* open error */
   /* process starts here */

   hsize(hshhdr,&rcdtot,&ovfltot,&emptytot,&freeblk);
   dellst = (unsigned char *)
      malloc(rcdtot * hshhdr->keylen);
   if (dellst == NULL)
      {
      fprintf(stderr,"delhsh: out of memory\n");
      exit(1);
      } /* out of mem */
   lstptr = (unsigned char *) dellst;
   position = SWPFRST;
   totdel = rslt = 0;
   while (!rslt)
      {
      rslt = hsweep(hshhdr,position,NULL);
      if (rslt == SWPEOF) break;
      if (rslt || hshhdr->reason)
	 {
	 fprintf(stderr,"delhsh: sweep "
	    "error %d reason %d\n",
	    rslt, hshhdr->reason);
	 exit(1);
	 } /* if sweep err */
      p = (unsigned char *) hshhdr->rcd
	 + hshhdr->keyofst;
      q = (unsigned char *) p + hshhdr->keylen;
      r = (unsigned char *) lstptr;
      while (p < q) *r++ = *p++;
      totdel++;
      lstptr += hshhdr->keylen;
      position = SWPNEXT;
      } /* for each sweep record */

   if (!totdel)
      {
      fprintf(stderr,"delhsh: empty database\n");
      exit(1);
      } /* nothing to delete */

   maxdel = totdel;
   delerr = 0;
   i = lmt;
   while (i--)
      {
      lstptr = (unsigned char *) dellst
	 + (rndnum(maxdel,rndSeed) * hshhdr->keylen);
      rslt = hdlet(hshhdr,lstptr);
      if (rslt == 1)  /* not found */
	 delerr++;
      else if (rslt)
	 {
	 fprintf(stderr,"delhsh: delete "
	    "error rslt %d reason %d\n",
	    rslt, hshhdr->reason);
	 exit(1);
	 } /* delete err */
      printf("%7d.  ",
	 lmt - i);
      /* print key */
      p = (unsigned char *) lstptr;
      q = (unsigned char *) p + hshhdr->keylen;
      while (p < q) putchar(*p++);
      putchar('\n');
      /* shift left */
      p = (unsigned char *) lstptr + hshhdr->keylen;
      q = (unsigned char *) dellst
	 + (totdel * hshhdr->keylen);
      r = (unsigned char *) lstptr;
      while (p < q) *r++ = *p++;
      maxdel--;
      } /* for each deleted record */


   hsize(hshhdr,&rcdtot,&ovfltot,&emptytot,&freeblk);
   printf("delhsh: rcds %d "
      "ovfl %d empty %d free %d errors %d\n",
      rcdtot, ovfltot, emptytot, freeblk,
      delerr);
   hchain(hshhdr,&count,&smlen,&smlensq,&freelen);
   if (hshhdr->reason)
      {
      fprintf(stderr,"delhsh: hchain "
	 "error reason %d\n",
	 hshhdr->reason);
      rslt = hdbck(hshhdr);
      if (rslt || hshhdr->reason)
         {
         fprintf(stderr,"chains: hchain "
	    "error %d reason %d\n",
	    rslt, hshhdr->reason);
         } /* if check err */
      exit(1);
      } /* if hchain err */
   printf("delhsh: totrcds %d count %d "
      "sum %d sumsq %d free %d\n",
      hshhdr->totrcds, count,
      smlen, smlensq, freelen);
   if (count)
      {
      avg = (double) smlen / (double) count;
      variance = (((double) count * (double) smlensq)
         - ((double) smlen * (double) smlen))
         / ((double) count * ((double) count - 1.0));
      stdev = sqrt(variance);
      printf("delhsh: average %.1f stdev %.1f stdev3 %.1f\n",
         avg, stdev, stdev * 3.0);
      } /* if not divide by zero */
   else
      {
      printf("delhsh: no statistics\n");
      } /* else divide by zero */
   /**********************************************************/
   hclose(hshhdr);
   return(0);
   } /* main */
